<?php

class application{

	public $autoload = null;
	
	public $config = null;

	public $session = null;

	public function __construct(){

		spl_autoload_register(array($this, 'autoload'));
		
		set_exception_handler(array('library_exception', 'handler'));
		
		set_error_handler(array($this, 'errorHandler'));
		
		register_shutdown_function(array($this,'shutdownFunction'));
		
		$this->config();
		
		session_start();
		
	}
	
	public function config(){
	
		date_default_timezone_set('America/Bogota');
		
		ini_set('default_charset','utf-8');
		
		setlocale(LC_ALL, 'es_ES.UTF-8');

		if(function_exists('mb_internal_encoding')):

			mb_internal_encoding('UTF-8');

		endif;
	
	}
	
	public static function autoload($classname){
	
		$classpath = str_replace('_', '/', $classname);

		try {

			$classfile = APP_PATH.DS.$classpath.'.php';
			
			if (file_exists($classfile)):

				require_once $classfile;
				
				return true;

			endif;

		}catch(Exception $e){

			library_exception::handler($e);

			exit;

		}

	}
	
	public static function errorHandler($code, $error, $file = null, $line = null){

		throw new ErrorException($error, $code, 0, $file, $line);

		return true;

	}
	
	public static function shutdownFunction(){

		$error = error_get_last();

		if($error):

			$exception = new ErrorException($error['message'], $error['type'], 0, $error['file'], $error['line']);
		
			library_exception::handler($exception);
	
		endif;

	}	

	public function init() {

		$request = $this->getPathInfo();
		
		$route_request = array();

		if ($request):
			
			$rewrite = new models_rewrite();

			if($rewrite->getpath($request)):
			
				$_request = 'postal/publica/id/'.$request;
				
				$route_request = explode("/", $_request);
			
			else:
			
				$route_request = explode("/", $request);
			
			endif;
			

		else:

			$route_request = array("index","index");

		endif;
		
		if (isset($route_request[0])):

			$this->route = array(
				"controller" => isset($route_request[0]) ? $route_request[0] : 'index',
				"action" => isset($route_request[1]) ? $route_request[1] : 'index'
			);

			$params = array();

			for($i = 2, $l = sizeof($route_request); $i < $l; $i += 2):

				$params[$route_request[$i]] = isset($route_request[$i + 1]) ? urldecode($route_request[$i + 1]) : '';

			endfor;

			$this->params = $params;

		else:

			$this->route = array(
				"controller" => "index",
				"action" => "notfound",
			);

		endif;

		$controller = 'controllers_' . $this->route['controller'];
		
		$action = $this->route['action'].'Action';
		
		if(class_exists($controller)):
		
			if(!method_exists($controller,$action)):
				
				$this->route = array(
					"module" => "core",
					"controller" => "index",
					"action" => "notfound",
				);
			
			endif;
		
		else:
		
				$this->route = array(
					"controller" => "index",
					"action" => "notfound",
				);

		endif;
		
		
		
		$controller = 'controllers_' . $this->route['controller'];
		
		$action = $this->route['action'].'Action';
				
		$reflectionclass = new ReflectionClass($controller);

		$controller = $reflectionclass->newInstance($this);

		$method = $reflectionclass->getMethod($action);

		$method->invoke($controller);

	}

	public function getAction() {

		$action_name = $this->route["action"]."Action";

		return $action_name;
	
	}
	
	public function getViewName(){

		$module = $this->route["module"];
		
		$controller = $this->route["controller"];

		$action = $this->route["action"];
		
		$view_name = $module . '_views_' . $controller.'_'.$action;
		
		return $view_name;	
	
	}
	
	public function getPathInfo(){

		$request_uri = $_SERVER['REQUEST_URI'];

		$request_pos = strpos($request_uri, '?');

		if ($request_pos):

			$request_uri = substr($request_uri, 0, $request_pos);

		endif;

		$base_url = $this->getBaseUrl();

		$path_info = substr($request_uri, strlen($base_url));

		$path_info = trim($path_info, '/');

		return $path_info;
		
	}

	public function getBaseUrl(){

		$script_name = $_SERVER['SCRIPT_NAME'];

		$base_url = rtrim(dirname($script_name), '/');

		return $base_url;

	}
	
	public function getWebUrl($https = false) {
	
		if($https):
		
			$protocol = 'https://';

		else:
			
			$protocol = 'http://';
			
		endif;
		
		$domain = isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : $_SERVER['SERVER_NAME'];

		$base_url = $this->getBaseUrl();

		$url = $protocol . $domain . $base_url . '/';

		return $url;

	}
	
	
	
	public function getParam($var, $default = null) {

		if (isset($this->params[$var])):

			$param = isset($this->params[$var]) ? $this->params[$var] : $default;

			return $param;

		endif;

		$param = isset($_GET[$var]) ? $_GET[$var] : $default;

		return $param;

	}
	
	public function paramsToUrl($params){

		$_params = '';

		foreach ($params as $key => $param):

			$_params .= $key . '/' . $param . '/';

		endforeach;

		return $_params;

	}

}